
#include "cmdlib.h"
#include "threads.h"
#include <sys/types.h>
#include <sys/sysctl.h>

#define MAX_THREADS	64

int getProcessorCount()
{
	int count;
	size_t size = sizeof( count );
	
	if( sysctlbyname("hw.ncpu",&count,&size,NULL,0) )
	{
		return 1;
	}
	
	if( count > MAX_THREADS )
	{
		count = MAX_THREADS;
	}
	
	return count;
}

#ifdef __alpha
int		numthreads = getProcessorCount();
pthread_mutex_t	*my_mutex;

#else
int		numthreads = 1;
#endif

void InitThreads()
{
#ifdef __alpha
	pthread_mutexattr_t	mattrib;

	my_mutex = (pthread_mutex_t*)malloc( sizeof(pthread_mutex_t) );
	
	if (pthread_mutexattr_init( &mattrib ) == -1)
	{
		Error ("pthread_mutexattr_init failed");
	}
	
	if (pthread_mutexattr_settype( &mattrib, PTHREAD_MUTEX_NORMAL ) == -1)
	{
		Error ("pthread_mutexattr_settype failed");
	}
	
	if (pthread_mutex_init( my_mutex, &mattrib ) == -1)
	{
		Error ("pthread_mutex_init failed");
	}
#endif
}

void* LightThread( void *junk );

/*
===============
RunThreadsOn
===============
*/
void RunThreadsOn()
{
#ifdef __alpha
	pthread_t	work_threads[MAX_THREADS];
	pthread_attr_t	attrib;
	int		i;
	
	if (numthreads == 1)
	{
		LightThread( NULL );
		return;
	}
	else
	{
		printf( "Using multithreading (%d threads)\n", numthreads );
	}
		
	if( pthread_attr_init( &attrib ) == -1 )
	{
		Error ("pthread_attr_init failed");
	}
	
	if (pthread_attr_setstacksize (&attrib, 0x100000) == -1)
	{
		Error ("pthread_attr_setstacksize failed");
	}
	
	for (i=0 ; i<numthreads ; i++)
	{
		if (pthread_create(&work_threads[i], &attrib, LightThread, NULL) == -1)
		{
			Error ("pthread_create failed");
		}
	}
		
	for (i=0 ; i<numthreads ; i++)
	{
		if (pthread_join (work_threads[i], NULL) == -1)
		{
			Error ("pthread_join failed");
		}
	}

	// Destroy the mutex
	
	if (pthread_mutex_destroy( my_mutex ) == -1)
	{
		Error ("pthread_mutex_destroy failed");
	}
	
#else
	LightThread( NULL );
#endif
}
